@@ -4,8 +4,12 @@ import math
 import trio
 import heapq
 import signal
+import sniffio
 import asyncio
 import warnings
+from contextvars import ContextVar
+from async_generator import asynccontextmanager
+from async_generator import async_generator, yield_
 
 from .handles import Handle, TimerHandle
 
@@ -25,6 +29,8 @@ __all__ = ['BaseTrioEventLoop', 'TrioExecutor']
 
 _mswindows = (sys.platform == "win32")
 
+_shim_running = ContextVar("shim_running", default=False)
+
 try:
     _wait_readable = trio.hazmat.wait_readable
     _wait_writable = trio.hazmat.wait_writable
@@ -209,39 +215,96 @@ class BaseTrioEventLoop(asyncio.SelectorEventLoop):
         This is a Trio coroutine.
         """
         self._check_closed()
+        t = sniffio.current_async_library_cvar.set("asyncio")
         coro = asyncio.ensure_future(coro, loop=self)
-        return await run_future(coro)
+        try:
+            return await run_future(coro)
+        finally:
+            sniffio.current_async_library_cvar.reset(t)
 
     def wrap_generator(self, gen, *args):
+        """Call an asyncio generator.
+
+        Deprecated: instead of
+
+            await loop.wrap_generator(gen, *args)
+
+        simply call
+
+            await loop.run_asyncio(gen(*args))
+        """
+        warnings.warn("Use 'run_asyncio(gen(*args))' instead'", DeprecationWarning)
+
         return run_generator(self, gen(*args))
 
     def run_iterator(self, aiter):
         """Call an asyncio iterator from Trio.
 
-        Example Usage::
-            async def slow_nums():
-                n = 0
-                while True:
-                    asyncio.sleep(1)
-                    yield n
-                    n += 1
+        Deprecated: instead of
 
-            async def trio_code(loop):
-                async for n in loop.run_iterator(slow_nums()):
-                    print(n)
+            await loop.run_iterator(iter)
+
+        simply call
+
+            await loop.run_asyncio(iter)
         """
+        warnings.warn("Use 'run_asyncio(iter)' instead'", DeprecationWarning)
+
         return run_generator(self, aiter)
 
-    async def run_asyncio(self, proc, *args):
+    @asynccontextmanager
+    @async_generator
+    async def wrap_trio_context(self, ctx):
+        """Run a Trio context manager from asyncio.
+        """
+        res = await self.run_trio(ctx.__aenter__)
+        try:
+            await yield_(res)
+        except BaseException as exc:
+            if not await self.run_trio(ctx.__aexit__, type(exc), exc, exc.__traceback__):
+                raise
+        else:
+            await self.run_trio(ctx.__aexit__, None, None, None)
+
+    def run_asyncio(self, proc, *args):
         """Run an asyncio function or method from Trio.
 
         :return: whatever the procedure returns.
         :raises: whatever the procedure raises.
 
-        This is a Trio coroutine.
+        This is (essentially) a Trio coroutine.
+
+        You can also use this for calling an asyncio-style
+        async iterator or async context manager.
         """
-        f = proc(*args)
-        return await self.run_coroutine(f)
+
+        class t2aWrapper:
+            def __await__(slf):
+                f = proc(*args)
+                return self.run_coroutine(f).__await__()
+
+            def __aenter__(slf):
+                proc_enter = getattr(proc, "__aenter__", None)
+                if proc_enter is None or args:
+                    raise RuntimeError(
+                        "Call 'run_asyncio(ctxfactory(*args))', not 'run_asyncio(ctxfactory, *args)'"
+                    )
+                f = proc_enter()
+                return self.run_coroutine(f)
+
+            def __aexit__(slf, *tb):
+                f = proc.__aexit__(*tb)
+                return self.run_coroutine(f)
+
+            def __aiter__(slf):
+                proc_iter = getattr(proc, "__anext__", None)
+                if proc_iter is None or args:
+                    raise RuntimeError(
+                        "Call 'run_asyncio(gen(*args))', not 'run_asyncio(gen, *args)'"
+                    )
+                return run_generator(self, proc)
+
+        return t2aWrapper()
 
     def run_trio(self, proc, *args):
         """Run an asynchronous Trio function from asyncio.
@@ -297,7 +360,7 @@ class BaseTrioEventLoop(asyncio.SelectorEventLoop):
             if scope.cancelled_caught:
                 f.cancel()
                 return
-        except Exception as exc:
+        except BaseException as exc:
             if not f.cancelled():  # pragma: no branch
                 f.set_exception(exc)
         else:
@@ -674,9 +737,10 @@ class BaseTrioEventLoop(asyncio.SelectorEventLoop):
 
         self._stopped.clear()
         task_status.started()
+        sniffio.current_async_library_cvar.set("asyncio")
 
         try:
-            while True:
+            while not self._stopped.is_set():
                 await self._main_loop_one()
         except StopAsyncIteration:
             # raised by .stop_me() to interrupt the loop
